==> abe-cli-0.2.7.tgz: package/src/cli/helpers/abe-utils.js <==
import extend from 'extend'
import loremIpsum from 'lorem-ipsum'
import clc from 'cli-color'
import fse from 'fs-extra'
import ajaxRequest from 'ajax-request'
import {Promise} from 'es6-promise'

import {
  config,
  cli,
  log,
  Sql,
  folderUtils,
  fileUtils,
  FileParser,
  fileAttr,
  getAttr
} from '../'

export default class Utils {

  constructor() {
    this._form = {
      default: {
        item:[]
      }
    }
    this._key = []
  }

  /**
   * Get all input from a template
   * @return {Array} array of input form
   */
  get form(){
    return this._form
  }

  /**
   * Check if key is not is the form array
   * @param  {[type]} key [description]
   * @return {[type]}     [description]
   */
  dontHaveKey(key){
    return typeof this._key[key] === "undefined" || this._key[key] === null
  }

  /**
   * Add entry to abe engine form
   * @param {String} type        textarea | text | meta | link | image | ...
   * @param {String} key         unique ID, no space allowed
   * @param {String} desc        input description
   * @param {Int}    maxLength   maximum characteres allowed inside input
   * @param {String} tab         tab name
   * @param {String} jsonValue   
   * @return {Void}
   */
  add(type, key, desc, maxLength, tab, jsonValue, source, display, reload) {
    var value = ''
    var block = ''
    if(typeof jsonValue !== 'undefined' && jsonValue !== null) {
      value = jsonValue
    }

    if(key.indexOf('[') < 0 && key.indexOf('.') > -1) {
      block = key.split('.')[0]
    }

    if(tab === '' || typeof tab === 'undefined' || tab === null) tab = 'default'
    else if(typeof this._form[tab] === 'undefined' || this._form[tab] === null) this._form[tab] = {item:[]}
    this._key[key] = true // save key for dontHaveKey()
    this._form[tab].item.push({
      type: type,
      key: key.replace(/\./, '-'),
      description: desc,
      maxLength: maxLength,
      placeholder: '',
      value: value,
      tab: tab,
      source: source,
      reload: (typeof reload !== 'undefined' && reload !== null && reload === 'true') ? true : false,
      display: display,
      block: block
    })
  }

  isBlock(str){
    return str.indexOf('[') < 0 && str.indexOf('.') > 0
  }

  /**
   * Test if a string don't contains string key from ABE block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is not a block content
   */
  isSingleAbe(str, text){
    return  !new RegExp('#each(.)+?' + getAttr(str, 'key').split('.')[0]).test(text) &&
            str.indexOf('{{#') < 0 &&
            str.indexOf('#each') < 0 &&
            str.indexOf('{{/') < 0 &&
            str.indexOf('/each') < 0 &&
            str.indexOf('attrAbe') < 0
  }

  /**
   * Test if a string contains string key from ABE block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is a block content
   */
  isBlockAbe(str) {
    return str.indexOf('abe') > -1 && getAttr(str, 'key').indexOf('.') > -1
  }

  /**
   * Test if a string contains string key from {{#each}} block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is a block content
   */
  isEachStatement(str) {
    return str.indexOf('#each') > -1 || str.indexOf('/each') > -1
  }

  /**
   * Encode / Escape && add data-abe attributs
   * @param  {String} block
   * @return {String} escaped string
   */
  encodeAbe(block){
    var matchAbe = block.match(/>\s*\{\{abe .*\}\}/g)
    if(matchAbe){
      for (var i = 0; i < matchAbe.length; i++)
        block = block.replace(
          matchAbe[i],
          ' data-abe="'  + getAttr(matchAbe[i], 'key').replace('.', '[0]-') + '" >'
        )
    }
    matchAbe = block.match(/( [A-Za-z0-9\-\_]+="*{{.*}})/g)
    if(matchAbe){
      for (var i = 0; i < matchAbe.length; i++)
        if(typeof matchAbe !== 'undefined' && matchAbe !== null){
          block = block.replace(
            matchAbe[i],
            ' data-abe-attr="'  + (matchAbe[i].split('=')[0]).trim() + '"' +
            ' data-abe="' + getAttr(matchAbe[i], 'key').replace('.', '[0]-') + '" ' + matchAbe[i]
          )
        }
    }
    return escape(block)
  }

  /**
   * Add some stuff like style / script before closing </body> tag
   * @param  {String} text html page
   * @return {String} text + some sugar stuff added on the fly
   */
  insertDebugtoolUtilities(text){
    return text.replace(
      /<\/body>/,
        `<style>
          body [data-abe]{ transition: box-shadow 600ms ease-in-out; box-shadow: 0; }
          body .select-border{ border-color: #007CDE; box-shadow: 0 3px 13px #7CBAEF; }
          body img.display-attr:before { content: attr(alt); }
          body a.display-attr:before { content: attr(title); }
          body .display-attr:before { position: absolute; display: block; z-index: 555; font-size: 10px; background-color: rgba(255, 255, 255, 0.75); padding: 2px 5px; color: #5D5D5D; }
          .hidden-abe{ display: none!important; }
        </style>
      </body>`
    )
  }

  lorem(type, v) {
    var lorem = ''
    if(type === 'text') {
      lorem = loremIpsum({
        units: 'sentences' // Generate words, sentences, or paragraphs.
        , sentenceLowerBound: 5
        , sentenceUpperBound: 10
      })
    }else if(type === 'link') {
      lorem = 'http://www.google.com'
    }else if(type === 'image' || type === 'file') {
      var width = getAttr(v, 'width')
      width = (width !== '') ? width : 300
      var height = getAttr(v, 'height')
      height = (height !== '') ? height : 300
      lorem = `http://placehold.it/${height}x${width}`
    }else if(type === 'textarea' || type === 'rich') {
      lorem = loremIpsum({
        units: 'paragraphs' // Generate words, sentences, or paragraphs.
        , paragraphLowerBound: 3
        , paragraphUpperBound: 7
      })
    }
    return lorem
  }

  static escapeRegExp(str) {
    var specials = [
      // order matters for these
      "-"
      , "["
      , "]"
      // order doesn't matter for any of these
      , "/"
      , "{"
      , "}"
      , "("
      , ")"
      , "*"
      , "+"
      , "?"
      , "."
      , "\\"
      , "^"
      , "$"
      , "|"
    ]

    // I choose to escape every character with '\'
    // even though only some strictly require it when inside of []
    , regex = RegExp('[' + specials.join('\\') + ']', 'g')
    return str.replace(regex, "\\$&");
  }

  static addMetas(tpl, json, type, obj = {}, user = {}) {
    let meta = config.meta.name

    json[meta] = extend({}, json[meta])
    var currentDate = new Date()
    var username = user.name || ''
    var abeUrl = (type === 'publish') ? json[meta].link : fileAttr.add(json[meta].link, 'd' + currentDate.toISOString()) + ''

    if(typeof json[meta].date === 'undefined' || json[meta].date === null) {
      json[meta].date = currentDate
      json[meta].author = username
    }
    json[meta].latest = {
      date: currentDate,
      author: username,
      abeUrl: abeUrl
    }

    if(typeof json[meta][type] === 'undefined' || json[meta][type] === null) {
      json[meta][type] = JSON.parse(JSON.stringify(obj))
      json[meta][type].date = currentDate
      json[meta][type].author = username
      json[meta][type].abeUrl = abeUrl
    }
    json[meta][type].latest = JSON.parse(JSON.stringify(obj))
    json[meta][type].latest.date = currentDate
    json[meta][type].latest.author = username
    json[meta][type].latest.abeUrl = abeUrl
  }

  static getDataList(tplPath, text, jsonPage) {
    var p = new Promise((resolve, reject) => {
      var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
          match

      var sourceAttr = config.source.name
      if(typeof jsonPage[sourceAttr] === 'undefined' || jsonPage[sourceAttr] === null) {
        jsonPage[sourceAttr] = {}
      }

      var promises = []
      while (match = listReg.exec(text)) {

        var pSource = new Promise((resolveSource, rejectSource) => {
          var source = getAttr(match[0], 'source')
          var key = getAttr(match[0], 'key')
          var type = Sql.getSourceType(source)
          switch (type) {
            case 'request':
              var data = Sql.getDataRequest(tplPath, match[0], jsonPage)
              jsonPage[sourceAttr][key] = data

              if(typeof jsonPage[key] !== 'undefined' && jsonPage[key] !== null) {
                var newJsonValue = []
                Array.prototype.forEach.call(jsonPage[key], (oldValue) => {
                  Array.prototype.forEach.call(jsonPage[sourceAttr][key], (newValue) => {
                    if(typeof oldValue[config.meta.name] !== 'undefined' && oldValue[config.meta.name] !== null
                      && oldValue[config.meta.name].link === newValue[config.meta.name].link) {
                      newJsonValue.push(newValue)
                    }
                  })
                })
                jsonPage[key] = newJsonValue
              }
              resolveSource()
              break;
            case 'value':
              var value = Sql.getDataSource(match[0])

              if(value.indexOf('{') > -1 || value.indexOf('[') > -1) {
                try{
                  value = JSON.parse(value)

                  jsonPage[sourceAttr][key] = value
                }catch(e){
                  jsonPage[sourceAttr][key] = null
                  console.log(clc.red(`Error ${value}/is not a valid JSON`),  `\n${e}`)
                }
              }
              resolveSource()
              break;
            case 'url':
              // @TODO
              ajaxRequest(source, (err, res, body) => {
                if(typeof body === 'string') {
                  var parsedBody = JSON.parse(body)
                  if(typeof parsedBody === 'object' && Object.prototype.toString.call(parsedBody) === '[object Array]') {
                    jsonPage[sourceAttr][key] = parsedBody
                  }else if(typeof parsedBody === 'object' && Object.prototype.toString.call(parsedBody) === '[object Object]') {
                    jsonPage[sourceAttr][key] = [parsedBody]
                  }
                }else if(typeof body === 'object' && Object.prototype.toString.call(body) === '[object Array]') {
                  jsonPage[sourceAttr][key] = body
                }else if(typeof body === 'object' && Object.prototype.toString.call(body) === '[object Object]') {
                  jsonPage[sourceAttr][key] = body
                }
                resolveSource()
              })

              break;
            case 'file':
              jsonPage[sourceAttr][key] = FileParser.getJson(fileUtils.concatPath(config.root, config.reference.url, source))
              resolveSource()
              break;
            default:
              resolveSource()
              break;
          }
        })
        promises.push(pSource)
      }

      Promise.all(promises)
        .then(() => {
          resolve()
        });
      // return filesRequest
      })

    return p
  }

  static removeDataList(text) {
    var listReg = /({{abe.*type=[\'|\"]data.*}})/g

    log.delAndWrite('Util-getList', text)
    return text.replace(listReg, '')
  }
}

==> abe-cli-0.2.13.tgz: package/src/cli/helpers/abe-utils.js <==
import extend from 'extend'
import loremIpsum from 'lorem-ipsum'
import clc from 'cli-color'
import fse from 'fs-extra'
import ajaxRequest from 'ajax-request'
import {Promise} from 'es6-promise'

import {
  config,
  cli,
  log,
  Sql,
  folderUtils,
  fileUtils,
  FileParser,
  fileAttr,
  getAttr
} from '../'

export default class Utils {

  constructor() {
    this._form = {
      default: {
        item:[]
      }
    }
    this._key = []
  }

  /**
   * Get all input from a template
   * @return {Array} array of input form
   */
  get form(){
    return this._form
  }

  /**
   * Check if key is not is the form array
   * @param  {[type]} key [description]
   * @return {[type]}     [description]
   */
  dontHaveKey(key){
    return typeof this._key[key] === "undefined" || this._key[key] === null
  }

  /**
   * Add entry to abe engine form
   * @param {String} type        textarea | text | meta | link | image | ...
   * @param {String} key         unique ID, no space allowed
   * @param {String} desc        input description
   * @param {Int}    maxLength   maximum characteres allowed inside input
   * @param {String} tab         tab name
   * @param {String} jsonValue   
   * @return {Void}
   */
  add(type, key, desc, maxLength, tab, jsonValue, source, display, reload, order) {
    var value = ''
    var block = ''
    if(typeof jsonValue !== 'undefined' && jsonValue !== null) {
      value = jsonValue
    }

    if(key.indexOf('[') < 0 && key.indexOf('.') > -1) {
      block = key.split('.')[0]
    }

    if(tab === '' || typeof tab === 'undefined' || tab === null) tab = 'default'
    else if(typeof this._form[tab] === 'undefined' || this._form[tab] === null) this._form[tab] = {item:[]}
    this._key[key] = true // save key for dontHaveKey()
    this._form[tab].item.push({
      type: type,
      key: key.replace(/\./, '-'),
      description: desc,
      maxLength: maxLength,
      placeholder: '',
      value: value,
      tab: tab,
      source: source,
      order: order,
      reload: (typeof reload !== 'undefined' && reload !== null && reload === 'true') ? true : false,
      display: display,
      block: block
    })
  }

  isBlock(str){
    return str.indexOf('[') < 0 && str.indexOf('.') > 0
  }

  /**
   * Test if a string don't contains string key from ABE block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is not a block content
   */
  isSingleAbe(str, text){
    return  !new RegExp('#each(.)+?' + getAttr(str, 'key').split('.')[0]).test(text) &&
            str.indexOf('{{#') < 0 &&
            str.indexOf('#each') < 0 &&
            str.indexOf('{{/') < 0 &&
            str.indexOf('/each') < 0 &&
            str.indexOf('attrAbe') < 0
  }

  /**
   * Test if a string contains string key from ABE block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is a block content
   */
  isBlockAbe(str) {
    return str.indexOf('abe') > -1 && getAttr(str, 'key').indexOf('.') > -1
  }

  /**
   * Test if a string contains string key from {{#each}} block statement
   * @param  {String}  str string to test
   * @return {Boolean} true = this is a block content
   */
  isEachStatement(str) {
    return str.indexOf('#each') > -1 || str.indexOf('/each') > -1
  }

  /**
   * Encode / Escape && add data-abe attributs
   * @param  {String} block
   * @return {String} escaped string
   */
  encodeAbe(block){
    var matchAbe = block.match(/>\s*\{\{abe .*\}\}/g)
    if(matchAbe){
      for (var i = 0; i < matchAbe.length; i++)
        block = block.replace(
          matchAbe[i],
          ' data-abe="'  + getAttr(matchAbe[i], 'key').replace('.', '[0]-') + '" >'
        )
    }
    matchAbe = block.match(/( [A-Za-z0-9\-\_]+="*{{.*}})/g)
    if(matchAbe){
      for (var i = 0; i < matchAbe.length; i++)
        if(typeof matchAbe !== 'undefined' && matchAbe !== null){
          block = block.replace(
            matchAbe[i],
            ' data-abe-attr="'  + (matchAbe[i].split('=')[0]).trim() + '"' +
            ' data-abe="' + getAttr(matchAbe[i], 'key').replace('.', '[0]-') + '" ' + matchAbe[i]
          )
        }
    }
    return escape(block)
  }

  /**
   * Add some stuff like style / script before closing </body> tag
   * @param  {String} text html page
   * @return {String} text + some sugar stuff added on the fly
   */
  insertDebugtoolUtilities(text){
    return text.replace(
      /<\/body>/,
        `<style>
          body [data-abe]{ transition: box-shadow 600ms ease-in-out; box-shadow: 0; }
          body .select-border{ border-color: #007CDE; box-shadow: 0 3px 13px #7CBAEF; }
          body img.display-attr:before { content: attr(alt); }
          body a.display-attr:before { content: attr(title); }
          body .display-attr:before { position: absolute; display: block; z-index: 555; font-size: 10px; background-color: rgba(255, 255, 255, 0.75); padding: 2px 5px; color: #5D5D5D; }
          .hidden-abe{ display: none!important; width: 0px !important; height: 0px!important; position: absolute; left: -10000px; top: -10000px; visibility: hidden;}
        </style>
      </body>`
    )
  }

  lorem(type, v) {
    var lorem = ''
    if(type === 'text') {
      lorem = loremIpsum({
        units: 'sentences' // Generate words, sentences, or paragraphs.
        , sentenceLowerBound: 5
        , sentenceUpperBound: 10
      })
    }else if(type === 'link') {
      lorem = 'http://www.google.com'
    }else if(type === 'image' || type === 'file') {
      var width = getAttr(v, 'width')
      width = (width !== '') ? width : 300
      var height = getAttr(v, 'height')
      height = (height !== '') ? height : 300
      lorem = `http://placehold.it/${height}x${width}`
    }else if(type === 'textarea' || type === 'rich') {
      lorem = loremIpsum({
        units: 'paragraphs' // Generate words, sentences, or paragraphs.
        , paragraphLowerBound: 3
        , paragraphUpperBound: 7
      })
    }
    return lorem
  }

  static escapeRegExp(str) {
    var specials = [
      // order matters for these
      "-"
      , "["
      , "]"
      // order doesn't matter for any of these
      , "/"
      , "{"
      , "}"
      , "("
      , ")"
      , "*"
      , "+"
      , "?"
      , "."
      , "\\"
      , "^"
      , "$"
      , "|"
    ]

    // I choose to escape every character with '\'
    // even though only some strictly require it when inside of []
    , regex = RegExp('[' + specials.join('\\') + ']', 'g')
    return str.replace(regex, "\\$&");
  }

  static addMetas(tpl, json, type, obj = {}, user = {}, date = null) {
    let meta = config.meta.name

    json[meta] = extend({}, json[meta])
    var currentDate = date || new Date()
    var username = user.name || ''
    var abeUrl = (type === 'publish') ? json[meta].link : fileAttr.add(json[meta].link, 'd' + currentDate.toISOString()) + ''

    if(typeof json[meta].date === 'undefined' || json[meta].date === null) {
      json[meta].date = currentDate
      json[meta].author = username
    }
    json[meta].latest = {
      date: currentDate,
      author: username,
      abeUrl: abeUrl
    }

    if(typeof json[meta][type] === 'undefined' || json[meta][type] === null) {
      json[meta][type] = JSON.parse(JSON.stringify(obj))
      json[meta][type].date = currentDate
      json[meta][type].author = username
      json[meta][type].abeUrl = abeUrl
    }
    json[meta][type].latest = JSON.parse(JSON.stringify(obj))
    json[meta][type].latest.date = currentDate
    json[meta][type].latest.author = username
    json[meta][type].latest.abeUrl = abeUrl
  }

  static getDataList(tplPath, text, jsonPage) {
    var p = new Promise((resolve, reject) => {
      var listReg = /({{abe.*type=[\'|\"]data.*}})/g,
          match

      var sourceAttr = config.source.name
      if(typeof jsonPage[sourceAttr] === 'undefined' || jsonPage[sourceAttr] === null) {
        jsonPage[sourceAttr] = {}
      }

      var promises = []
      while (match = listReg.exec(text)) {

        var pSource = new Promise((resolveSource, rejectSource) => {
          var source = getAttr(match[0], 'source')
          var key = getAttr(match[0], 'key')
          var type = Sql.getSourceType(source)
          switch (type) {
            case 'request':
              var data = Sql.getDataRequest(tplPath, match[0], jsonPage)
              jsonPage[sourceAttr][key] = data

              if(typeof jsonPage[key] !== 'undefined' && jsonPage[key] !== null) {
                var newJsonValue = []
                Array.prototype.forEach.call(jsonPage[key], (oldValue) => {
                  Array.prototype.forEach.call(jsonPage[sourceAttr][key], (newValue) => {
                    if(typeof oldValue[config.meta.name] !== 'undefined' && oldValue[config.meta.name] !== null
                      && oldValue[config.meta.name].link === newValue[config.meta.name].link) {
                      newJsonValue.push(newValue)
                    }
                  })
                })
                jsonPage[key] = newJsonValue
              }
              resolveSource()
              break;
            case 'value':
              var value = Sql.getDataSource(match[0])

              if(value.indexOf('{') > -1 || value.indexOf('[') > -1) {
                try{
                  value = JSON.parse(value)

                  jsonPage[sourceAttr][key] = value
                }catch(e){
                  jsonPage[sourceAttr][key] = null
                  console.log(clc.red(`Error ${value}/is not a valid JSON`),  `\n${e}`)
                }
              }
              resolveSource()
              break;
            case 'url':
              // @TODO
              ajaxRequest(source, (err, res, body) => {
                if(typeof body === 'string') {
                  var parsedBody = JSON.parse(body)
                  if(typeof parsedBody === 'object' && Object.prototype.toString.call(parsedBody) === '[object Array]') {
                    jsonPage[sourceAttr][key] = parsedBody
                  }else if(typeof parsedBody === 'object' && Object.prototype.toString.call(parsedBody) === '[object Object]') {
                    jsonPage[sourceAttr][key] = [parsedBody]
                  }
                }else if(typeof body === 'object' && Object.prototype.toString.call(body) === '[object Array]') {
                  jsonPage[sourceAttr][key] = body
                }else if(typeof body === 'object' && Object.prototype.toString.call(body) === '[object Object]') {
                  jsonPage[sourceAttr][key] = body
                }
                resolveSource()
              })

              break;
            case 'file':
              jsonPage[sourceAttr][key] = FileParser.getJson(fileUtils.concatPath(config.root, source))
              resolveSource()
              break;
            default:
              resolveSource()
              break;
          }
        })
        promises.push(pSource)
      }

      Promise.all(promises)
        .then(() => {
          resolve()
        });
      // return filesRequest
      })

    return p
  }

  static removeDataList(text) {
    var listReg = /({{abe.*type=[\'|\"]data.*}})/g

    log.delAndWrite('Util-getList', text)
    return text.replace(listReg, '')
  }
}
